/*
 * File: FileReader.c
 *
 * Author: M. I. Karavelas
 * Class:  EM101 / Fall Semester 2008
 */

#include <stdio.h>
#include <assert.h>
#include <ctype.h>
#include "simpio.h"
#include "strlib.h"
#include "FileReader.h"


/* static variables (variables vivisble within the scope of this file) */
static FILE* fp = NULL;
static bool isFileOpen = FALSE;
static string buffer;


/*
 * Function: RemoveSpaces
 * Usage: str2 = RemoveSpaces(str1)
 * --------------------------------
 * Removes leading and trailing white space characters from the given
 * string.
 * This function is visible only within the scope of this file.
 */

static string RemoveSpaces(string line)
{
  if ( line == NULL ) { return NULL; }

  int len = StringLength(line), l, r;

  for (l = 0; l < len; l++) {
    if ( !isspace(IthChar(line, l)) ) break;
  }

  if ( l == len ) { return ""; }

  for (r = len - 1; r >= 0; r--) {
    if ( !isspace(IthChar(line, r)) ) break;
  }

  return SubString(line, l, r);
}


/*
 * Function: IsValidWord
 * Usage: if ( IsValidWord(word) )
 * -------------------------------
 * Returns TRUE if the given word is valid, i.e., it consists solely
 * of printable characters except the space character.
 * This function is visible only within the scope of this file.
 */

static bool IsValidWord(string word)
{
  int len = StringLength(word), i;

  for (i = 0; i != len; i++) {
    char c = IthChar(word, i);
    if ( !isprint(c) || c == ' ' ) { return FALSE; }
  }
  return TRUE;
}


/*
 * Function: OpenFile
 * Usage: if ( OpenFile(fname) )
 * -----------------------------
 * Opens the given file for reading and initializes the static
 * variable buffer with the first word in the file.
 */

bool OpenFile(string filename)
{
  if ( isFileOpen ) {
    fclose(fp);
    isFileOpen = FALSE;
  }

  fp = fopen(filename, "r");

  if ( !fp ) { return FALSE; }

  isFileOpen = TRUE;
  do {
    buffer = RemoveSpaces(ReadLine(fp));
  } while ( buffer != NULL && StringEqual(buffer, "") );

  return TRUE;
}


/*
 * Function: GetNextWord
 * Usage: word = GetNextWord();
 * ----------------------------
 * Returns the next word in the file. This is the word currently
 * stored in the buffer. The contents of the buffer are properly
 * updated. An error message is generated if the word read is not
 * valid; a word is considered to be valid if it consists of printable
 * ASCII characters except the space character.
 */

string GetNextWord(void)
{
  string word = buffer;

  if ( !IsValidWord(word) ) {
    Error("The word \"%s\" is not a valid word.\n", word);
  }

  do {
    buffer = RemoveSpaces(ReadLine(fp));
  } while ( buffer != NULL && StringEqual(buffer, "") );

  return word;
}


/*
 * Function: AtEndOfFile
 * Usage: if ( AtEndOfFile() )
 * ---------------------------
 * Return TRUE if we have reached the end of the file opened with the
 * OpenFile function, FALSE otherwise.
 * This is equivalent to checking whether buffer is equal to NULL
 * since the function ReadLine() returns NULL (see functions OpenFile
 * and GetNextWord) if and only if we have reached the end of the
 * file.
 */

bool AtEndOfFile(void)
{
  return (buffer == NULL);
}
